-- Author      : Barogio, KarlThePagan, Onilink, Moonrydre and Bleric all have authored the addon. Latest revision by Potdisc.
-- Create Date : 8/9/2012 11:08:28 PM

--[[ TODO:
	Check if IsInRaid() works in Battleground.
]]
Gag = LibStub("AceAddon-3.0"):NewAddon("Gag", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0")
-- GLOBALS: Gag, gag_data

local _G = _G
local pairs = pairs
local tsort = table.sort
local tinsert = table.insert
local strchar = string.char
local strlen = string.len
local time = time
local strsub = strsub
local strlower = strlower
local strupper = strupper
local loadstring = loadstring
local tContains = tContains

local CreateFrame = CreateFrame
local UIParent = UIParent
local PlaySoundFile = PlaySoundFile
local GetUnitName = GetUnitName
local DoEmote = DoEmote
local SendChatMessage = SendChatMessage
local UnitIsGroupLeader = UnitIsGroupLeader
local IsInRaid = IsInRaid
local IsInGroup = IsInGroup

local LE_PARTY_CATEGORY_HOME = LE_PARTY_CATEGORY_HOME
local LE_PARTY_CATEGORY_INSTANCE = LE_PARTY_CATEGORY_INSTANCE

local L_ToggleDropDownMenu = L_ToggleDropDownMenu
local L_UIDropDownMenu_Initialize = L_UIDropDownMenu_Initialize
local L_UIDropDownMenu_AddButton = L_UIDropDownMenu_AddButton
-- GLOBALS: L_UIDROPDOWNMENU_MENU_VALUE

local defaults;
local db; -- DB shortening
local LastEmoteTime = time();
local gag_data_sorted_keys = {};
local MenuFrame;
local LastDropdownMenuValue;


 local GagLDB = {
	type = "launcher",
	icon = "Interface\\Icons\\ACHIEVEMENT_GUILDPERK_MRPOPULARITY",
	label = "Gag",
	OnClick = function(this, button)
		L_ToggleDropDownMenu(1, nil, MenuFrame, this, 0, 0);
	end,
	OnTooltipShow = function(tooltip)
		tooltip:AddLine("Gag");
		tooltip:AddLine("Click to open a list of available songs, sounds and options!")
	end,
};

 -- Options table
 local options = {
	type = 'group',
	args = {
		enable = {
			order = 1,
			type = 'execute',
			name = 'Enable',
			desc = "Enables Gag emotes and sounds for your own character",
			func = function() Gag:Enable() end,
		},
		disable = {
			order = 2,
			type = 'execute',
		    name = 'Disable',
			desc = "Disables Gag emotes and sounds for your own character",
			func = function() Gag:Disable() end,
		},
		enableEmotes = {
			order = 3,
			type = "toggle",
			name = "Toggle Emotes",
			desc = "Toggle your own emotes on/off",
			get = function() return db.EmoteEnabled; end,
			set = function() db.EmoteEnabled = not db.EmoteEnabled end,
		},
		enableGroup = {
			order = 4,
			type = 'toggle',
			name = 'Toggle Group',
			desc = "Toggles Gag emotes on/off for the entire group if you are a party, raid, or battleground leader",
			get = function() return db.GroupEnabled; end,
			set = function() Gag:ToggleGroup() end,
		},
		list = {
			order = 5,
			type = 'execute',
			name = 'List',
			desc = "List all available Gag slash commands",
			func = function() Gag:ListEmotes() end,
		},
		ping = {
			order = 6,
			type = 'execute',
			name = 'Ping',
			desc = "Pings your group to see who has Gag installed.  Older versions of Gag will not be seen.",
			func = function() Gag:Ping() end,
		},
	},
 }

 function Gag:OnInitialize()
	self:RegisterComm("Gag")
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Gag", options, {"gag"})

	defaults = {
		profile = {
			IsEnabled = true,
			GroupEnabled = true,
			EmoteEnabled = true,
			SayEnabled = false,
			LDBIconStorage = {}, -- LibDBIcon storage
		}
	 }
	self.db = LibStub("AceDB-3.0"):New("GagDB", defaults, true)
	db = self.db.profile

	LibStub("LibDataBroker-1.1"):NewDataObject("GagMinimapButton", GagLDB);
	LibStub("LibDBIcon-1.0"):Register("GagMinimapButton", GagLDB, db.LDBIconStorage);

	MenuFrame = CreateFrame("FRAME", "GagMenu", UIParent, "L_UIDropDownMenuTemplate")
	L_UIDropDownMenu_Initialize(MenuFrame, self.DropDown_OnLoad, "MENU")
 end

 function Gag:OnEnable()
	db.IsEnabled = true
	if not gag_data then
		self:Print("you have no soundpacks installed! Visit the Gag page on http://www.curse.com/addons/wow/gag to find soundpacks.")
	else
		--Generate slash commands from gag_data
		for key,value in pairs(gag_data) do
			_G.SlashCmdList["GAG_"..key] = loadstring("Gag:SayGagKey(\""..key.."\");");
			_G["SLASH_GAG_"..key.."1"] = "/"..key;
		end

		--Fill the array
		local i = 0;
		for key,value in pairs(gag_data) do
			gag_data_sorted_keys[i] = key;
			i = i + 1;
		end
		tsort(gag_data_sorted_keys)
	 end
 end

function Gag:OnDisable()
	db.IsEnabled = false;
	self:Print("disabled")
end


function Gag:ToggleGroup()
	if UnitIsGroupLeader("player") then
		db.GroupEnabled = not db.GroupEnabled
		if db.GroupEnabled then
			Gag:Send("GroupEnable")
			self:Print("Enabled for the entire group");
		else
			Gag:Send("GroupDisable")
			self:Print("Disabled for the entire group");
		end
	else
		self:Print("You are not the group leader");
	end
end

function Gag:Send(msg, to)
	if to then
		self:SendCommMessage("Gag", msg, "WHISPER", to)
	else
		if IsInRaid() then
			self:SendCommMessage("Gag", msg, (not IsInRaid(LE_PARTY_CATEGORY_HOME) and IsInRaid(LE_PARTY_CATEGORY_INSTANCE)) and "INSTANCE_CHAT" or "RAID");
		elseif IsInGroup() then
			self:SendCommMessage("Gag", msg, (not IsInGroup(LE_PARTY_CATEGORY_HOME) and IsInGroup(LE_PARTY_CATEGORY_INSTANCE)) and "INSTANCE_CHAT" or "PARTY");
		end
	end
end

function Gag:Ping()
	Gag:Send("PingSend")
	self:Print("Players in your group with Gag installed:");
end

function Gag:GROUP_ROSTER_UPDATE()
	if UnitIsGroupLeader("player") then
		if(db.GroupEnabled) then
			Gag:Send("GroupEnable");
		else
			Gag:Send("GroupDisable")
		end
	end
end

function Gag:ListEmotes()
	self:Print("List of sounds/commands:");
	--Output the array
	local i = 1;
	local line = "";
		for key,value in pairs(gag_data_sorted_keys) do
		line = line..value.." ";
		if i == 6 then
			self:Print(line);
			i = 0;
			line = "";
		end
		i = i + 1;
	end
end

function Gag:SayGagKey(key)
	if(db.IsEnabled) then
		if(db.GroupEnabled) then
			if(time() - LastEmoteTime > 1) then
				local emote = gag_data[key];
				if (emote["msg"] ~= nil) and db.EmoteEnabled then SendChatMessage(emote["msg"], "EMOTE"); end
				if (emote["text"] ~= nil) and db.SayEnabled then SendChatMessage(emote["text"], "SAY"); end
				Gag:DoEmote(key, true);
			else
				self:Print("Emote blocked to prevent spam");
			end
		else
			self:Print("Disabled by your group leader");
		end
	else
		self:Print("Disabled.  Type \"/gag enable\" to enable this mod.");
	end
end

function Gag:DoEmote(key, arg2)
	if (db.IsEnabled) then
		if (db.GroupEnabled) then
			if (time() - LastEmoteTime > 1) then
				local emote = gag_data[key]
				if (emote["emote"] ~= nil) and db.EmoteEnabled then DoEmote(emote["emote"]); end
				if (emote["file"] ~= nil) then PlaySoundFile(emote["file"], "Master"); end
				if arg2 then
					Gag:Send(key)
				end
				LastEmoteTime = time();
			end
		else
		self:Print("Disabled by your group leader");
		end
	end
end

function Gag:OnCommReceived(prefix, msg, distri, sender)
	if prefix == "Gag" then
		--sender = Ambiguate(sender, "none") Ace Ambiguates for us
		if sender == GetUnitName("player") then return end; -- don't do anything if we send the message
		if msg == "GroupEnable" then
			db.GroupEnabled = true

		elseif msg == "GroupDisable" then
			db.GroupEnabled = false

		elseif msg == "PingSend" then
			Gag:Send("PingReply", sender)

		elseif msg == "PingReply" then
			self:Print(sender)

		else
			Gag:DoEmote(msg, false)
		end
	end
end

function Gag.DropDown_OnLoad(self, level)
	if gag_data then
		local emote;
		local text = ""
		if level == 1 then
			local c;
			local categories = {}
			local packs = {}
			categories[0] = "Etc";
			for key, data in pairs(gag_data) do
				--get categories
				emote = gag_data[key]
				if emote["category"] ~= nil then c= emote["category"]; else c = "Etc"; end
				local alreadyInCategories = false
				for _, v in pairs(categories) do
					if c == v then alreadyInCategories = true; end
				end
				if not alreadyInCategories then tinsert(categories, c); end
				-- get packs
				if not tContains(packs, data["pack"]) then tinsert(packs, data["pack"]); end
			end
			L_UIDropDownMenu_AddButton({text = "Gag", isTitle = true, notCheckable = true, disabled = true}, level);
			L_UIDropDownMenu_AddButton({text = "", notCheckable = true, disabled = true}, level);
			L_UIDropDownMenu_AddButton({text = "All", value = "All", hasArrow = true, notCheckable = true}, level);
			-- add categories
			for _, value in pairs(categories) do
				L_UIDropDownMenu_AddButton({text = value, value = value, hasArrow = true, notCheckable = true}, level);
			end
			-- add packs, if any
			if #packs > 0 then
				L_UIDropDownMenu_AddButton({text = "", notCheckable = true, disabled = true}, level);
				L_UIDropDownMenu_AddButton({text = "Packs", isTitle = true, notCheckable = true, disabled = true}, level);
				for _, value in pairs(packs) do
					L_UIDropDownMenu_AddButton({text = value, value = value, hasArrow = true, notCheckable = true}, level);
				end
			end

			L_UIDropDownMenu_AddButton({text = "", notCheckable = true, disabled = true}, level);
			L_UIDropDownMenu_AddButton({text = "Options", isTitle = true, notCheckable = true, disabled = true}, level);
			L_UIDropDownMenu_AddButton({text = "Emotes Enabled", checked = db.EmoteEnabled, func = function() db.EmoteEnabled = not db.EmoteEnabled; end, tooltipTitle = "Toggle Emotes", tooltipText = "Toggle your own emotes on/off", keepShownOnClick = true}, level);
			L_UIDropDownMenu_AddButton({text = "Say Enabled", checked = db.SayEnabled, func = function() db.SayEnabled = not db.SayEnabled; end, tooltipTitle = "Toggle Say", tooltipText = "Toggle to print in /say on/off", keepShownOnClick = true}, level);
			if UnitIsGroupLeader("player") then -- only allow for group changes if the player is the group leader
				L_UIDropDownMenu_AddButton({text = "Group Enabled", func = function() Gag:ToggleGroup(); end, checked = db.GroupEnabled, tooltipTitle = "Toggle Group Emotes", tooltipText = "Toggle emotes and sounds on/off for the entire group if you're the group leader.", keepShownOnClick = true }, level);
			end
			L_UIDropDownMenu_AddButton({text = "Gag Enabled", checked = db.IsEnabled, func = function() db.IsEnabled = not db.IsEnabled end, tooltipTitle = "Enable", tooltipText = "Turn Gag on/off.", keepShownOnClick = true}, level);

		elseif level == 2 then
			local value = L_UIDROPDOWNMENU_MENU_VALUE
			if value == "All" then
				for i = 65,90 do
					local c = strchar(i);
					L_UIDropDownMenu_AddButton({text = c, notCheckable = true, hasArrow = true}, level);
				end
			else
				local subcategories = {}
				local letters = {}
				-- build subcategories or packs, and get the letters for the rest
				for k, emote in pairs(gag_data) do
					local add = true
					if emote["pack"] == value and emote["category"] then
						add = false
						if not tContains(subcategories, emote["category"]) then	tinsert(subcategories, emote["category"]);end
					elseif emote["category"] == value and emote["subcategory"] then
						add = false
						if not tContains(subcategories, emote["subcategory"]) then tinsert(subcategories, emote["subcategory"]);end
					end
					if add and not tContains(letters, strsub(strupper(k),1,1)) and (value == emote["category"] or value == emote["pack"]) then tinsert(letters, strsub(strupper(k),1,1)) end
				end

				--output subcategories
				tsort(subcategories)
				for k,v in pairs(subcategories) do
					L_UIDropDownMenu_AddButton({text = v, value = v, notCheckable = true, hasArrow = true}, level)
				end

				-- output letters
				tsort(letters)
				for _,c in pairs(letters) do
					L_UIDropDownMenu_AddButton({text = c, value = c, notCheckable = true, hasArrow = true}, level)
				end
			end
			LastDropdownMenuValue = value
		elseif level == 3 then
			local value = L_UIDROPDOWNMENU_MENU_VALUE
			if strlen(value) == 1 then
				for _,key in pairs(gag_data_sorted_keys) do
					emote = gag_data[key]
					if strlower(strsub(key, 1, 1)) == strlower(value) then
						if not emote["subcategory"] and LastDropdownMenuValue == emote ["category"] or LastDropdownMenuValue == "All" or LastDropdownMenuValue == emote["pack"] then
							text = "/"..key.." - |cFF999999"..emote["text"].."|r";
							L_UIDropDownMenu_AddButton({text = text, notCheckable = true, func = function() Gag:SayGagKey(key) end}, level)
						end
					end
				end
			else
				tsort(gag_data_sorted_keys)
				for _,key in pairs(gag_data_sorted_keys) do
					emote = gag_data[key]
					if emote["category"] == value and LastDropdownMenuValue == emote["pack"] and emote["subcategory"] then
						-- create pack subcategory
						local subcategories = {}
						for _,v in pairs(gag_data) do
							if v["pack"] == emote["pack"] and not tContains(subcategories, v["subcategory"]) then tinsert(subcategories, v["subcategory"]); end
						end
						--output subcategories
						for k,v in pairs(subcategories) do
							L_UIDropDownMenu_AddButton({text = v, value = v, notCheckable = true, hasArrow = true}, level)
						end
					elseif emote["subcategory"] == value or emote["category"] == value and LastDropdownMenuValue == emote["pack"] then
						text = "/"..key.." - |cFF999999"..emote["text"].."|r";
						L_UIDropDownMenu_AddButton({text = text, notCheckable = true, func = function() Gag:SayGagKey(key) end}, level)
					end
				end
			end
		elseif level == 4 then -- pack subcategory
			local value = L_UIDROPDOWNMENU_MENU_VALUE
			tsort(gag_data_sorted_keys)
				for _,key in pairs(gag_data_sorted_keys) do
					emote = gag_data[key]
					if emote["subcategory"] == value and LastDropdownMenuValue == emote["pack"] then
						text = "/"..key.." - |cFF999999"..emote["text"].."|r";
						L_UIDropDownMenu_AddButton({text = text, notCheckable = true, func = function() Gag:SayGagKey(key) end}, level)
					end
				end
		end
	end
end
